//
//  FeedListViewModel.swift
//  ReviewsFeed
//
//  Created by Javier Cicchelli on 18/03/2024.
//  Copyright © 2024 Röck+Cöde. All rights reserved.
//

import Foundation
import ReviewsFilterKit
import ReviewsiTunesKit

extension FeedListViewController {
    final class ViewModel: ObservableObject {
        
        // MARK: Type aliases
        typealias Configuration = FeedListViewController.Configuration
        
        // MARK: Constants
        private let configuration: Configuration
        
        private let filterWords: FilterWordsUseCase = .init()
        private let topWords: TopWordsUseCase = .init()

        // MARK: Properties
        @Published var filter: FilterOption = .all
        @Published var isFilterEnabled: Bool = false
        @Published var isFiltering: Bool = false
        @Published var isLoading: Bool = false

        var items: [Review] = []
        var words: [TopWord] = []
        
        private var reviewsAll: [Review] = []
        private var reviewsFiltered: FilteredReviews = [:]
        private var reviewsTopWords: TopWordsReviews = [:]
        
        lazy private var iTunesService: iTunesService = {
            .init(configuration: .init(session: configuration.session))
        }()

        // MARK: Initialisers
        init(configuration: Configuration = .init()) {
            self.configuration = configuration
        }
        
        // MARK: Computed
        var itemsCount: Int {
            isWordsShowing
                ? items.count
                : items.count + 1
        }
        
        var isWordsShowing: Bool {
            filter != .all
            && !words.isEmpty
        }

        // MARK: Functions
        func fetch() {
            Task {
                isFilterEnabled = false
                isLoading = items.isEmpty

                do {
                    let output = try await iTunesService.getReviews(.init(
                        appID: configuration.appID,
                        countryCode: configuration.countryCode
                    ))
                    
                    reviewsAll = output.reviews.map(Review.init)
                    reviewsFiltered = FilterOption.allCases
                        .reduce(into: FilteredReviews()) { partialResult, option in
                            partialResult[option] = reviewsAll.filter { $0.rating.stars == option.rawValue }
                        }
                    reviewsTopWords = reviewsFiltered
                        .mapValues { reviews in
                            reviews.map(\.comment)
                                .compactMap { try? filterWords($0) }
                        }
                        .mapValues {
                            topWords($0).map(TopWord.init)
                        }

                    items = filter == .all
                        ? reviewsAll
                        : reviewsFiltered[filter] ?? []
                    
                    isFilterEnabled = !items.isEmpty
                } catch {
                    // TODO: handle this error gracefully.
                    print("ERROR: \(error.localizedDescription)")
                }

                isLoading = false
            }
        }
        
        func filter(by option: FilterOption) {
            guard option != filter else { return }
            
            items = option == .all
                ? reviewsAll
                : reviewsFiltered[option] ?? []
            words = reviewsTopWords[option] ?? []

            filter = option
        }
        
        func item(for index: Int) -> Review? {
            guard index < items.count else { return nil }
            
            return isWordsShowing
                ? items[index - 1]
                : items[index]
        }

    }
}

// MARK: - Helpers
private extension FeedListViewController.ViewModel {
    
    // MARK: Type aliases
    typealias FilteredReviews = [FilterOption: [Review]]
    typealias TopWordsReviews = [FilterOption: [TopWord]]

}
