//
//  FilterWordsUseCase.swift
//  ReviewsFilterKit
//
//  Created by Javier Cicchelli on 17/03/2024.
//  Copyright © 2024 Röck+Cöde VoF. All rights reserved.
//

import Foundation
import ReviewsFoundationKit

public struct FilterWordsUseCase {
    
    // MARK: Type aliases
    public typealias Input = String
    public typealias Output = [WordCount]
    
    // MARK: Initialisers
    public init() {}
    
    // MARK: Functions
    public func callAsFunction(_ input: Input) throws -> Output {
        let regularExpression = try NSRegularExpression(
            pattern: .Pattern.wordsLongerThan4Characters,
            options: .caseInsensitive
        )
        
        let matches = regularExpression.matches(
            in: input,
            options: [],
            range: .init(input.startIndex..., in: input)
        )
        
        guard !matches.isEmpty else { return [] }
        
        let terms = matches.compactMap {
            Range($0.range, in: input).map { String(input[$0]) }
        }

        let wordsAll = terms
                .map { ($0, $0.folding(options: .caseInsensitive, locale: nil)) }
                .map(Word.init)

        let wordsCount = wordsAll.map { word -> WordCount in
            .init(
                word: word,
                count: terms.filter { $0 == word.term }.count
            )
        }.reduce(into: [WordCount]()) { partialResult, wordCount in
            guard partialResult.filter({ $0 == wordCount }).isEmpty else { return }

            partialResult.append(wordCount)
        }
        
        let wordsUnique = Dictionary(grouping: wordsCount) { wordCount -> String in
            wordsAll
                .filter { $0.token == wordCount.word.token }
                .map(\.token)
                .first ?? .empty
        }.map { (key, values) -> WordCount in
            .init(
                word: .init(
                    term: key.capitalized,
                    token: key
                ),
                count: values
                    .map(\.count)
                    .reduce(0, +)
            )
        }

        return wordsUnique.sorted {
            guard $0.count != $1.count else {
                return $0.word.token < $1.word.token
            }
            
            return $0.count > $1.count
        }
    }
    
}

// MARK: - Helpers
private extension FilterWordsUseCase {
    
    // MARK: Functions
    func compareWords(
        _ lword: String,
        _ rword: String
    ) -> Bool {
        lword.compare(rword, options: [
            .caseInsensitive,
            .diacriticInsensitive
        ]) == .orderedSame
    }
    
}

// MARK: - String+Constants
private extension String {
    enum Pattern {
        static let wordsLongerThan4Characters = "\\w{4,}"
    }
}
