//
//  FilterWordsUseCaseTests.swift
//  ReviewsFilterTest
//
//  Created by Javier Cicchelli on 18/03/2024.
//  Copyright © 2024 Röck+Cöde VoF. All rights reserved.
//

import ReviewsFilterKit
import ReviewsFoundationKit
import XCTest

final class FilterWordsUseCaseTests: XCTestCase {

    // MARK: Properties
    private var input: FilterWordsUseCase.Input!
    private var output: FilterWordsUseCase.Output!
    private var sut: FilterWordsUseCase!

    // MARK: Setup
    override func setUp() async throws {
        sut = .init()
    }

    // MARK: Function tests
    func testCallAsFunction_withInput_hasSomeLongWords() throws {
        // GIVEN
        input = "one two three four five six seven eight nine ten"
        
        // WHEN
        output = try sut(input)
        
        // THEN
        XCTAssertFalse(output.isEmpty)
        XCTAssertEqual(output, [
            .init(word: .init(term: "Eight", token: "eight"), count: 1),
            .init(word: .init(term: "Five", token: "five"), count: 1),
            .init(word: .init(term: "Four", token: "four"), count: 1),
            .init(word: .init(term: "Nine", token: "nine"), count: 1),
            .init(word: .init(term: "Seven", token: "seven"), count: 1),
            .init(word: .init(term: "Three", token: "three"), count: 1),
        ])
    }
    
    func testCallAsFunction_withInput_hasSomeRepeatedLongWords() throws {
        // GIVEN
        input = "one two three three four five five six seven eight nine nine ten"
        
        // WHEN
        output = try sut(input)
        
        // THEN
        XCTAssertFalse(output.isEmpty)
        XCTAssertEqual(output, [
            .init(word: .init(term: "Five", token: "five"), count: 2),
            .init(word: .init(term: "Nine", token: "nine"), count: 2),
            .init(word: .init(term: "Three", token: "three"), count: 2),
            .init(word: .init(term: "Eight", token: "eight"), count: 1),
            .init(word: .init(term: "Four", token: "four"), count: 1),
            .init(word: .init(term: "Seven", token: "seven"), count: 1),
        ])
    }
    
    func testCallAsFunction_withInput_hasSomeRepeatedCaseSensitiveLongWords() throws {
        // GIVEN
        input = "one two three ThReE four FIVE five six seven eight NiNe nInE ten"
        
        // WHEN
        output = try sut(input)
        
        // THEN
        XCTAssertFalse(output.isEmpty)
        XCTAssertEqual(output, [
            .init(word: .init(term: "Five", token: "five"), count: 2),
            .init(word: .init(term: "Nine", token: "nine"), count: 2),
            .init(word: .init(term: "Three", token: "three"), count: 2),
            .init(word: .init(term: "Eight", token: "eight"), count: 1),
            .init(word: .init(term: "Four", token: "four"), count: 1),
            .init(word: .init(term: "Seven", token: "seven"), count: 1),
        ])
    }
    
    func testCallAsFunction_withInput_hasSomeDiacriticSensitiveLongWords() throws {
        // GIVEN
        input = "one two three Thrèé four FiVe FIVE six Šëvêń seven eight niñe nine ten"
        
        // WHEN
        output = try sut(input)
        
        // THEN
        XCTAssertFalse(output.isEmpty)
        XCTAssertEqual(output, [
            .init(word: .init(term: "Five", token: "five"), count: 2),
            .init(word: .init(term: "Eight", token: "eight"), count: 1),
            .init(word: .init(term: "Four", token: "four"), count: 1),
            .init(word: .init(term: "Nine", token: "nine"), count: 1),
            .init(word: .init(term: "Niñe", token: "niñe"), count: 1),
            .init(word: .init(term: "Seven", token: "seven"), count: 1),
            .init(word: .init(term: "Three", token: "three"), count: 1),
            .init(word: .init(term: "Thrèé", token: "thrèé"), count: 1),
            .init(word: .init(term: "Šëvêń", token: "šëvêń"), count: 1),
        ])
    }
    
    func testCallAsFunction_withInput_hasOnlyShortWords() throws {
        // GIVEN
        input = "no one is a two"
        
        // WHEN
        output = try sut(input)
        
        // THEN
        XCTAssertTrue(output.isEmpty)
        XCTAssertEqual(output, [])
    }

    func testCallAsFunction_withEmptyInput() throws {
        // GIVEN
        input = .empty

        // WHEN
        output = try sut(input)

        // THEN
        XCTAssertTrue(output.isEmpty)
        XCTAssertEqual(output, [])
    }
    
}
